local T = Deadened_T()
Deadened_T = nil

Deadened = AceLibrary("AceAddon-2.0"):new("AceHook-2.1", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0", "Sink-1.0", "Parser-3.0")
Deadened:RegisterDB("DeadenedDB", "DeadenedDBPC", "profile")
Deadened:RegisterDefaults('profile', {
	handlers = {},
	tankmode = false,
	sink10OutputSink = "None",
	announce = "None"
})

local handlers = {}
local announceModes = {"None", "Group", "Party", "Raid", "Say", "Yell"}

function Deadened:OnInitialize()
	self:CheckLocale()
end

function Deadened:OnEnable()
	self:RegisterEvent("Ace2_AddonEnabled")
	self:RegisterEvent("Ace2_AddonDisabled")
	self:RegisterParserEvent({ eventType = "Interrupt", sourceID = "player" }, "parseInterrupt")
	
	local options = {
		type = "group",
		args = {
			tankmode = {
				type = "toggle",
				name = "Tank mode",
				desc = "Suppress Spirit Shock instead of Deaden",
				get = function()
					return Deadened.db.profile.tankmode
				end,
				set = function(v)
					Deadened.db.profile.tankmode = v
				end
			},
			announce = {
				type = "text",
				name = "Announce",
				desc = "Announce interrupts to...",
				get = function()
					return Deadened.db.profile.announce
				end,
				set = function(v)
					Deadened.db.profile.announce = v
				end,
				validate = announceModes
			},
			modules = {
				type = "group",
				name = "Modules",
				desc = "Modules",
				args = {}
			}
		}
	}
	
	for k, v in pairs(handlers) do
		if self.db.profile.handlers[k] ~= false then
			v:Hook(true)
		end
		options.args.modules.args[k] = {
			type = "toggle", 
			name = k, 
			desc = "Toggle " .. k, 
			get = function()
				return Deadened.db.profile.handlers[k] ~= false or false
			end,
			set = function(v)
				Deadened.db.profile.handlers[k] = v
				if v then
					handlers[k]:Hook(true)
				else
					handlers[k]:Unhook(true)
				end
			end
		}
	end	
	Deadened:RegisterChatCommand({"/deaden", "/deadened"}, options)	
end

local function getDistribution(p)
	if p:lower() == "party" then
		if GetNumPartyMembers() > 0 then return "PARTY" end
	elseif p:lower() == "raid" then
		if GetNumRaidMembers() > 0 then return "RAID" end		
	elseif p:lower() == "group" then
		if GetNumRaidMembers() > 0 then return "RAID" end
		if GetNumPartyMembers() > 0 then return "PARTY" end
	elseif p:lower() == "say" then
		return "SAY"
	elseif p:lower() == "yell" then
		return "YELL"
	end
	return nil
end

function Deadened:parseInterrupt(info)
	if self.db.profile.announce == "None" then return end
	local d = getDistribution(self.db.profile.announce)
	if d then
		local msg = ("%s %s's %s!"):format(SPELL_FAILED_INTERRUPTED, info.recipientName, info.abilityName)
		SendChatMessage(msg, d)
	end	
end

function Deadened:OnDisable()
	self:UnhookAll()
end

function Deadened:CheckLocale()
	if T[GetLocale()] == nil then
		error("No localization for Deadened in " .. GetLocale())
	end
end

function Deadened:IsDeaden(v)
	self:CheckLocale()
	if self.db.profile.tankmode then
		return T[GetLocale()]["Spirit Shock"] == v
	else
		return T[GetLocale()]["Deaden"] == v
	end
end

function Deadened:Ace2_AddonEnabled(module)
	for k, v in pairs(handlers) do
		v:Hook(module)
	end
end

function Deadened:Ace2_AddonDisabled(module, first)
	for k, v in pairs(handlers) do
		v:Unhook(module)
	end
end

function Deadened:UNIT_SPELLCAST_START(orig_object, unit)
	if unit == "target" then
		local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
		if not Deadened:IsDeaden(displayName) then
			for k,v in pairs(self.hooks[orig_object]) do
				v(orig_object, unit)
			end
		else
			local spell, rank, displayName, icon, startTime, endTime = UnitCastingInfo(unit)
			self:Alert(("%s casting %s (%s) sec"):format(UnitName(unit), displayName, ("%2.1f"):format((endTime-startTime) / 1000 )))
		end
	end	
end

function Deadened:Alert(s)
	self:ScheduleEvent("Deaden-Alert", self.Pour, 0.25, self, s)
end

function Deadened:AddHandler(name, handler)	
	handlers[name] = handler
end
